@@ -19,17 +19,19 @@
  */
 
 #include <stdlib.h>
+#include <string.h>
 
 #include <glib.h>
 #include <glib-object.h>
 
-#include "girepository.h"
 #include "gtypelib.h"
+#include "ginfo.h"
 
 struct _GIBaseInfo 
 {
   gint type;
   gint ref_count;
+  GIRepository *repository;
   GIBaseInfo *container;
 
   GTypelib *typelib;
@@ -134,13 +136,16 @@ struct _GIUnionInfo
 
 /* info creation */
 GIBaseInfo *
-g_info_new (GIInfoType     type,
-	    GIBaseInfo    *container,
-	    GTypelib     *typelib, 
-	    guint32        offset)
+g_info_new_full (GIInfoType     type,
+		 GIRepository  *repository,
+		 GIBaseInfo    *container,
+		 GTypelib     *typelib, 
+		 guint32        offset)
 {
   GIBaseInfo *info;
 
+  g_return_val_if_fail (container != NULL || repository != NULL, NULL);
+
   info = g_new0 (GIBaseInfo, 1);
 
   info->ref_count = 1;
@@ -152,38 +157,48 @@ g_info_new (GIInfoType     type,
   if (container)
     info->container = g_base_info_ref (container);
 
+  info->repository = g_object_ref (repository);
+
   return info;
 }
 
+GIBaseInfo *
+g_info_new (GIInfoType     type,
+	    GIBaseInfo    *container,
+	    GTypelib     *typelib, 
+	    guint32        offset)
+{
+  return g_info_new_full (type, container->repository, container, typelib, offset);
+}
+
 static GIBaseInfo *
-g_info_from_entry (GTypelib *typelib,
+g_info_from_entry (GIRepository *repository,
+		   GTypelib *typelib,
 		   guint16    index)
 {
   GIBaseInfo *result;
   DirEntry *entry = g_typelib_get_dir_entry (typelib, index);
-  
+
   if (entry->local)
-    result = g_info_new (entry->blob_type, NULL, typelib, entry->offset);
-  else 
+    result = g_info_new_full (entry->blob_type, repository, NULL, typelib, entry->offset);
+  else
     {
       const gchar *namespace = g_typelib_get_string (typelib, entry->offset);
       const gchar *name = g_typelib_get_string (typelib, entry->name);
-      
-      GIRepository *repository = g_irepository_get_default ();
-      
+
       result = g_irepository_find_by_name (repository, namespace, name);
       if (result == NULL)
 	{
 	  GIUnresolvedInfo *unresolved;
 
 	  unresolved = g_new0 (GIUnresolvedInfo, 1);
-	  
+
 	  unresolved->type = GI_INFO_TYPE_UNRESOLVED;
 	  unresolved->ref_count = 1;
 	  unresolved->container = NULL;
 	  unresolved->name = name;
 	  unresolved->namespace = namespace;
-	  
+
 	  return (GIBaseInfo*)unresolved;
 	}
       return result;
@@ -212,6 +227,8 @@ g_base_info_unref (GIBaseInfo *info)
       if (info->container)
 	g_base_info_unref (info->container);
 
+      g_object_unref (info->repository);
+
       g_free (info);
     }
 }
@@ -476,7 +493,8 @@ g_function_info_get_flags (GIFunctionInfo *info)
   
   flags = 0;
 
-  if (base->container != NULL)
+  /* Make sure we don't flag Constructors as methods */
+  if (!blob->constructor && !blob->is_static)
     flags = flags | GI_FUNCTION_IS_METHOD;
     
   if (blob->constructor)
@@ -491,6 +509,9 @@ g_function_info_get_flags (GIFunctionInfo *info)
   if (blob->wraps_vfunc)
     flags = flags | GI_FUNCTION_WRAPS_VFUNC;
 
+  if (blob->throws)
+    flags = flags | GI_FUNCTION_THROWS;
+
   return flags;
 }
 
@@ -548,7 +569,7 @@ g_type_info_new (GIBaseInfo    *container,
   SimpleTypeBlob *type = (SimpleTypeBlob *)&typelib->data[offset];
 
   return (GITypeInfo *) g_info_new (GI_INFO_TYPE_TYPE, container, typelib, 
-				    type->reserved == 0 ? offset : type->offset);
+				    (type->reserved == 0 && type->reserved2 == 0) ? offset : type->offset);
 }
 
 /**
@@ -703,7 +724,7 @@ g_arg_info_may_be_null (GIArgInfo *info)
   GIBaseInfo *base = (GIBaseInfo *)info;
   ArgBlob *blob = (ArgBlob *)&base->typelib->data[base->offset];
   
-  return blob->null_ok;
+  return blob->allow_none;
 }
 
 GITransfer
@@ -720,12 +741,39 @@ g_arg_info_get_ownership_transfer (GIArgInfo *info)
     return GI_TRANSFER_NOTHING;
 }
 
+GIScopeType
+g_arg_info_get_scope (GIArgInfo *info)
+{
+  GIBaseInfo *base = (GIBaseInfo *)info;
+  ArgBlob *blob = (ArgBlob *)&base->typelib->data[base->offset];
+
+  return blob->scope;
+}
+
+gint
+g_arg_info_get_closure (GIArgInfo *info)
+{
+  GIBaseInfo *base = (GIBaseInfo *)info;
+  ArgBlob *blob = (ArgBlob *)&base->typelib->data[base->offset];
+
+  return blob->closure;
+}
+
+gint
+g_arg_info_get_destroy (GIArgInfo *info)
+{
+  GIBaseInfo *base = (GIBaseInfo *)info;
+  ArgBlob *blob = (ArgBlob *)&base->typelib->data[base->offset];
+
+  return blob->destroy;
+}
+
 GITypeInfo *
 g_arg_info_get_type (GIArgInfo *info)
 {
   GIBaseInfo *base = (GIBaseInfo *)info;
  
-  return g_type_info_new (base, base->typelib, base->offset + 8);
+  return g_type_info_new (base, base->typelib, base->offset + 12);
 }
 
 /* GITypeInfo functions */
@@ -735,7 +783,7 @@ g_type_info_is_pointer (GITypeInfo *info)
   GIBaseInfo *base = (GIBaseInfo *)info;
   SimpleTypeBlob *type = (SimpleTypeBlob *)&base->typelib->data[base->offset];
   
-  if (type->reserved == 0)
+  if (type->reserved == 0 && type->reserved2 == 0)
     return type->pointer;
   else
     {
@@ -751,7 +799,7 @@ g_type_info_get_tag (GITypeInfo *info)
   GIBaseInfo *base = (GIBaseInfo *)info;
   SimpleTypeBlob *type = (SimpleTypeBlob *)&base->typelib->data[base->offset];
 
-  if (type->reserved == 0)
+  if (type->reserved == 0 && type->reserved2 == 0)
     return type->tag;
   else
     {
@@ -768,7 +816,7 @@ g_type_info_get_param_type (GITypeInfo *info,
   GIBaseInfo *base = (GIBaseInfo *)info;
   SimpleTypeBlob *type = (SimpleTypeBlob *)&base->typelib->data[base->offset];
   
-  if (type->reserved != 0)
+  if (!(type->reserved == 0 && type->reserved2 == 0))
     {
       ParamTypeBlob *param = (ParamTypeBlob *)&base->typelib->data[base->offset];
 
@@ -794,12 +842,12 @@ g_type_info_get_interface (GITypeInfo *info)
   GIBaseInfo *base = (GIBaseInfo *)info;
   SimpleTypeBlob *type = (SimpleTypeBlob *)&base->typelib->data[base->offset];
   
-  if (type->reserved != 0)
+  if (!(type->reserved == 0 && type->reserved2 == 0))
     {
       InterfaceTypeBlob *blob = (InterfaceTypeBlob *)&base->typelib->data[base->offset];
       
       if (blob->tag == GI_TYPE_TAG_INTERFACE)
-	return g_info_from_entry (base->typelib, blob->interface);
+	return g_info_from_entry (base->repository, base->typelib, blob->interface);
     }
 
   return NULL;
@@ -811,7 +859,7 @@ g_type_info_get_array_length (GITypeInfo *info)
   GIBaseInfo *base = (GIBaseInfo *)info;
   SimpleTypeBlob *type = (SimpleTypeBlob *)&base->typelib->data[base->offset];
   
-  if (type->reserved != 0)
+  if (!(type->reserved == 0 && type->reserved2 == 0))
     {
       ArrayTypeBlob *blob = (ArrayTypeBlob *)&base->typelib->data[base->offset];
 
@@ -825,13 +873,33 @@ g_type_info_get_array_length (GITypeInfo *info)
   return -1;
 }
 
+gint
+g_type_info_get_array_fixed_size (GITypeInfo *info)
+{
+  GIBaseInfo *base = (GIBaseInfo *)info;
+  SimpleTypeBlob *type = (SimpleTypeBlob *)&base->typelib->data[base->offset];
+  
+  if (!(type->reserved == 0 && type->reserved2 == 0))
+    {
+      ArrayTypeBlob *blob = (ArrayTypeBlob *)&base->typelib->data[base->offset];
+
+      if (blob->tag == GI_TYPE_TAG_ARRAY)
+	{
+	  if (blob->has_size)
+	    return blob->size;
+	}
+    }
+
+  return -1;
+}
+
 gboolean
 g_type_info_is_zero_terminated (GITypeInfo *info)
 {
   GIBaseInfo *base = (GIBaseInfo *)info;
   SimpleTypeBlob *type = (SimpleTypeBlob *)&base->typelib->data[base->offset];
   
-  if (type->reserved != 0)
+  if (!(type->reserved == 0 && type->reserved2 == 0))
     {
       ArrayTypeBlob *blob = (ArrayTypeBlob *)&base->typelib->data[base->offset];
 
@@ -848,7 +916,7 @@ g_type_info_get_n_error_domains (GITypeInfo *info)
   GIBaseInfo *base = (GIBaseInfo *)info;
   SimpleTypeBlob *type = (SimpleTypeBlob *)&base->typelib->data[base->offset];
   
-  if (type->reserved != 0)
+  if (!(type->reserved == 0 && type->reserved2 == 0))
     {
       ErrorTypeBlob *blob = (ErrorTypeBlob *)&base->typelib->data[base->offset];
 
@@ -866,12 +934,13 @@ g_type_info_get_error_domain (GITypeInfo *info,
   GIBaseInfo *base = (GIBaseInfo *)info;
   SimpleTypeBlob *type = (SimpleTypeBlob *)&base->typelib->data[base->offset];
   
-  if (type->reserved != 0)
+  if (!(type->reserved == 0 && type->reserved2 == 0))
     {
       ErrorTypeBlob *blob = (ErrorTypeBlob *)&base->typelib->data[base->offset];
 
       if (blob->tag == GI_TYPE_TAG_ERROR)
-	return (GIErrorDomainInfo *) g_info_from_entry (base->typelib, 
+	return (GIErrorDomainInfo *) g_info_from_entry (base->repository,
+							base->typelib,
 							blob->domains[n]);
     }
 
@@ -895,7 +964,8 @@ g_error_domain_info_get_codes (GIErrorDomainInfo *info)
   GIBaseInfo *base = (GIBaseInfo *)info;
   ErrorDomainBlob *blob = (ErrorDomainBlob *)&base->typelib->data[base->offset];
   
-  return (GIInterfaceInfo *) g_info_from_entry (base->typelib, blob->error_codes);
+  return (GIInterfaceInfo *) g_info_from_entry (base->repository,
+						base->typelib, blob->error_codes);
 }
 
 
@@ -992,9 +1062,9 @@ g_registered_type_info_get_g_type (GIRegisteredTypeInfo *info)
     return G_TYPE_NONE;
   
   get_type_func = NULL;
-  if (!g_module_symbol (((GIBaseInfo*)info)->typelib->module,
-                        type_init,
-                        (void**) &get_type_func))
+  if (!g_typelib_symbol (((GIBaseInfo*)info)->typelib,
+                         type_init,
+                         (void**) &get_type_func))
     return G_TYPE_NONE;
   
   return (* get_type_func) ();
@@ -1087,6 +1157,24 @@ g_struct_info_find_method (GIStructInfo *info,
   return find_method (base, offset, blob->n_methods, name);
 }
 
+gsize
+g_struct_info_get_size (GIStructInfo *info)
+{
+  GIBaseInfo *base = (GIBaseInfo *)info;
+  StructBlob *blob = (StructBlob *)&base->typelib->data[base->offset];
+
+  return blob->size;
+}
+
+gsize
+g_struct_info_get_alignment (GIStructInfo *info)
+{
+  GIBaseInfo *base = (GIBaseInfo *)info;
+  StructBlob *blob = (StructBlob *)&base->typelib->data[base->offset];
+
+  return blob->alignment;
+}
+
 gint
 g_enum_info_get_n_values (GIEnumInfo *info)
 {
@@ -1109,6 +1197,28 @@ g_enum_info_get_value (GIEnumInfo *info,
   return (GIValueInfo *) g_info_new (GI_INFO_TYPE_VALUE, base, base->typelib, offset);
 }
 
+/**
+ * g_enum_info_get_storage_type:
+ * @info: GIEnumInfo
+ *
+ * Gets the tag of the type used for the enum in the C ABI. This will
+ * will be a signed or unsigned integral type.
+
+ * Note that in the current implementation the width of the type is
+ * computed correctly, but the signed or unsigned nature of the type
+ * may not match the sign of the type used by the C compiler.
+ *
+ * Return Value: the storage type for the enumeration
+ */
+GITypeTag
+g_enum_info_get_storage_type (GIEnumInfo *info)
+{
+  GIBaseInfo *base = (GIBaseInfo *)info;
+  EnumBlob *blob = (EnumBlob *)&base->typelib->data[base->offset];
+
+  return blob->storage_type;
+}
+
 /* GIObjectInfo functions */
 GIObjectInfo *
 g_object_info_get_parent (GIObjectInfo *info)
@@ -1117,11 +1227,20 @@ g_object_info_get_parent (GIObjectInfo *info)
   ObjectBlob *blob = (ObjectBlob *)&base->typelib->data[base->offset];
 
   if (blob->parent)
-    return (GIObjectInfo *) g_info_from_entry (base->typelib, blob->parent);
+    return (GIObjectInfo *) g_info_from_entry (base->repository,
+					       base->typelib, blob->parent);
   else
     return NULL;
 }
 
+gboolean
+g_object_info_get_abstract (GIObjectInfo    *info)
+{
+  GIBaseInfo *base = (GIBaseInfo *)info;
+  ObjectBlob *blob = (ObjectBlob *)&base->typelib->data[base->offset];
+  return blob->abstract != 0;
+}
+
 const gchar *
 g_object_info_get_type_name (GIObjectInfo *info)
 {
@@ -1156,7 +1275,8 @@ g_object_info_get_interface (GIObjectInfo *info,
   GIBaseInfo *base = (GIBaseInfo *)info;
   ObjectBlob *blob = (ObjectBlob *)&base->typelib->data[base->offset];
 
-  return (GIInterfaceInfo *) g_info_from_entry (base->typelib, blob->interfaces[n]);
+  return (GIInterfaceInfo *) g_info_from_entry (base->repository,
+						base->typelib, blob->interfaces[n]);
 }
 
 gint
@@ -1364,7 +1484,8 @@ g_interface_info_get_prerequisite (GIInterfaceInfo *info,
   GIBaseInfo *base = (GIBaseInfo *)info;
   InterfaceBlob *blob = (InterfaceBlob *)&base->typelib->data[base->offset];
 
-  return g_info_from_entry (base->typelib, blob->prerequisites[n]);
+  return g_info_from_entry (base->repository,
+			    base->typelib, blob->prerequisites[n]);
 }
 
 
@@ -1679,7 +1800,7 @@ g_constant_info_get_value (GIConstantInfo *info,
   ConstantBlob *blob = (ConstantBlob *)&base->typelib->data[base->offset];
 
   /* FIXME non-basic types ? */
-  if (blob->type.reserved == 0)
+  if (blob->type.reserved == 0 && blob->type.reserved2 == 0)
     {
       if (blob->type.pointer)
 	value->v_pointer = g_memdup (&base->typelib->data[blob->offset], blob->size);
@@ -1720,6 +1841,9 @@ g_constant_info_get_value (GIConstantInfo *info,
 	    case GI_TYPE_TAG_DOUBLE:
 	      value->v_double = *(gdouble*)&base->typelib->data[blob->offset];
 	      break;
+	    case GI_TYPE_TAG_TIME_T:
+	      value->v_long = *(long*)&base->typelib->data[blob->offset];
+	      break;
 	    case GI_TYPE_TAG_INT:
 	      value->v_int = *(gint*)&base->typelib->data[blob->offset];
 	      break;
@@ -1835,3 +1959,36 @@ g_union_info_get_discriminator (GIUnionInfo *info,
 
   return NULL;
 }
+
+GIFunctionInfo *
+g_union_info_find_method (GIUnionInfo *info,
+                          const gchar *name)
+{
+  gint offset;
+  GIBaseInfo *base = (GIBaseInfo *)info;
+  Header *header = (Header *)base->typelib->data;
+  UnionBlob *blob = (UnionBlob *)&base->typelib->data[base->offset];
+
+  offset = base->offset + header->union_blob_size
+    + blob->n_fields * header->field_blob_size;
+
+  return find_method (base, offset, blob->n_functions, name);
+}
+
+gsize
+g_union_info_get_size (GIUnionInfo *info)
+{
+  GIBaseInfo *base = (GIBaseInfo *)info;
+  UnionBlob *blob = (UnionBlob *)&base->typelib->data[base->offset];
+
+  return blob->size;
+}
+
+gsize
+g_union_info_get_alignment (GIUnionInfo *info)
+{
+  GIBaseInfo *base = (GIBaseInfo *)info;
+  UnionBlob *blob = (UnionBlob *)&base->typelib->data[base->offset];
+
+  return blob->alignment;
+}
