@@ -35,11 +35,6 @@ typedef struct {
 #define ALIGN_VALUE(this, boundary) \
   (( ((unsigned long)(this)) + (((unsigned long)(boundary)) -1)) & (~(((unsigned long)(boundary))-1)))
 
-#if !defined(__Darwin__)
-#  undef SHLIB_SUFFIX
-#  define SHLIB_SUFFIX G_MODULE_SUFFIX 
-#endif
-
 static void
 push_context (ValidateContext *ctx, const char *name)
 {
@@ -157,14 +152,22 @@ g_typelib_check_sanity (void)
 		  n, sizeof (s));					\
       size_check_ok = FALSE; \
     }
+
+  /* When changing the size of a typelib structure, you are required to update
+   * the hardcoded size here.  Do NOT change these to use sizeof(); these
+   * should match whatever is defined in the text specification and serve as
+   * a sanity check on structure modifications.
+   *
+   * Everything else in the code however should be using sizeof().
+   */
   
   CHECK_SIZE (Header, 108);
   CHECK_SIZE (DirEntry, 12);
   CHECK_SIZE (SimpleTypeBlob, 4);
-  CHECK_SIZE (ArgBlob, 12);
+  CHECK_SIZE (ArgBlob, 16);
   CHECK_SIZE (SignatureBlob, 8);
   CHECK_SIZE (CommonBlob, 8);
-  CHECK_SIZE (FunctionBlob, 16);
+  CHECK_SIZE (FunctionBlob, 20);
   CHECK_SIZE (InterfaceTypeBlob, 4);
   CHECK_SIZE (ArrayTypeBlob, 8);
   CHECK_SIZE (ParamTypeBlob, 4);
@@ -173,7 +176,7 @@ g_typelib_check_sanity (void)
   CHECK_SIZE (ValueBlob, 12);
   CHECK_SIZE (FieldBlob, 12);
   CHECK_SIZE (RegisteredTypeBlob, 16);
-  CHECK_SIZE (StructBlob, 20);
+  CHECK_SIZE (StructBlob, 24);
   CHECK_SIZE (EnumBlob, 20);
   CHECK_SIZE (PropertyBlob, 12);
   CHECK_SIZE (SignalBlob, 12);
@@ -182,7 +185,7 @@ g_typelib_check_sanity (void)
   CHECK_SIZE (InterfaceBlob, 28);
   CHECK_SIZE (ConstantBlob, 20);
   CHECK_SIZE (AnnotationBlob, 12);
-  CHECK_SIZE (UnionBlob, 28);
+  CHECK_SIZE (UnionBlob, 32);
 #undef CHECK_SIZE
 
   g_assert (size_check_ok);
@@ -283,7 +286,7 @@ validate_header (ValidateContext  *ctx,
       
     }
 
-  if (header->major_version != 1 || header->minor_version != 0)
+  if (header->major_version != 2 || header->minor_version != 0)
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
@@ -312,11 +315,11 @@ validate_header (ValidateContext  *ctx,
     }
 
   if (header->entry_blob_size != 12 ||
-      header->function_blob_size != 16 ||
+      header->function_blob_size != 20 ||
       header->callback_blob_size != 12 ||
       header->signal_blob_size != 12 ||
       header->vfunc_blob_size != 16 ||
-      header->arg_blob_size != 12 ||
+      header->arg_blob_size != 16 ||
       header->property_blob_size != 12 ||
       header->field_blob_size != 12 ||
       header->value_blob_size != 12 ||
@@ -325,10 +328,10 @@ validate_header (ValidateContext  *ctx,
       header->annotation_blob_size != 12 ||
       header->signature_blob_size != 8 ||
       header->enum_blob_size != 20 ||
-      header->struct_blob_size != 20 ||
+      header->struct_blob_size != 24 ||
       header->object_blob_size != 32 ||
       header->interface_blob_size != 28 ||
-      header->union_blob_size != 28)
+      header->union_blob_size != 32)
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
@@ -728,7 +731,7 @@ validate_function_blob (ValidateContext *ctx,
       g_set_error (error,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_BLOB,
-		   "Wrong blob type");
+		   "Wrong blob type %d, expected function", blob->blob_type);
       return FALSE;
     }
 
