@@ -20,4 +20,4 @@
 
 DATADIR = "${prefix}/share"
 DATADIR = DATADIR.replace(
-    "${prefix}", "/home/jdahlin/.local")
+    "${prefix}", "/home/johan/.local")
