@@ -2,20 +2,20 @@
 # GObject-Introspection - a framework for introspecting GObject libraries
 # Copyright (C) 2008  Johan Dahlin
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 2 of the License, or (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-# 02110-1301, USA.
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library; if not, write to the
+# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+# Boston, MA 02111-1307, USA.
 #
 
 from xml.etree.cElementTree import parse
@@ -94,8 +94,9 @@ class GIRParser(object):
         assert ns is not None
         self._namespace = Namespace(ns.attrib['name'],
                                     ns.attrib['version'])
-        self._shared_libraries.extend(
-            ns.attrib['shared-library'].split(','))
+        if 'shared-library' in ns.attrib:
+            self._shared_libraries.extend(
+                ns.attrib['shared-library'].split(','))
 
         parser_methods = {
             _corens('alias'): self._parse_alias,
@@ -150,8 +151,12 @@ class GIRParser(object):
         ctor_args.append(node.attrib.get(_cns('type')))
         for iface in node.findall(_corens('implements')):
             obj.interfaces.append(iface.attrib['name'])
+        for iface in node.findall(_corens('prerequisites')):
+            obj.prerequisities.append(iface.attrib['name'])
         for method in node.findall(_corens('method')):
-            obj.methods.append(self._parse_function_common(method, Function))
+            func = self._parse_function_common(method, Function)
+            func.is_method = True
+            obj.methods.append(func)
         for ctor in node.findall(_corens('constructor')):
             obj.constructors.append(
                 self._parse_function_common(ctor, Function))
@@ -211,8 +216,10 @@ class GIRParser(object):
                                      node.attrib[_glibns('get-type')],
                                      node.attrib.get(_cns('type')))
         else:
+            disguised = node.attrib.get('disguised') == '1'
             struct = Struct(node.attrib['name'],
-                            node.attrib.get(_cns('type')))
+                            node.attrib.get(_cns('type')),
+                            disguised=disguised)
         self._add_node(struct)
 
         if self._include_parsing:
@@ -281,8 +288,9 @@ class GIRParser(object):
         if self._include_parsing:
             return
         for method in node.findall(_corens('method')):
-            obj.methods.append(
-                self._parse_function_common(method, Function))
+            func = self._parse_function_common(method, Function)
+            func.is_method = True
+            obj.methods.append(func)
         for ctor in node.findall(_corens('constructor')):
             obj.constructors.append(
                 self._parse_function_common(ctor, Function))
