@@ -2,40 +2,51 @@
 # GObject-Introspection - a framework for introspecting GObject libraries
 # Copyright (C) 2008  Johan Dahlin
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 2 of the License, or (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-# 02110-1301, USA.
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library; if not, write to the
+# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+# Boston, MA 02111-1307, USA.
 #
 
 import os
+import sys
 import re
-import ctypes
-from ctypes.util import find_library
+import tempfile
+import shutil
+import subprocess
 
-from . import cgobject
-from .ast import (Callback, Constant, Enum, Function, Member, Namespace,
-                  Parameter, Property, Return, Struct, Type, Alias,
-                  Union, Field, type_name_from_ctype,
-                  default_array_types, TYPE_UINT8, PARAM_DIRECTION_IN)
+from .ast import (Alias, Bitfield, Callback, Constant, Enum, Function, Member,
+                  Namespace, Parameter, Property, Record, Return, Type, Union,
+                  Field, type_name_from_ctype,
+                  default_array_types, TYPE_UINT8, PARAM_TRANSFER_FULL)
 from .transformer import Names
 from .glibast import (GLibBoxed, GLibEnum, GLibEnumMember, GLibFlags,
                       GLibInterface, GLibObject, GLibSignal, GLibBoxedStruct,
                       GLibBoxedUnion, GLibBoxedOther, type_names)
-from .utils import extract_libtool, to_underscores, to_underscores_noprefix
+from .utils import to_underscores, to_underscores_noprefix
 
 default_array_types['guchar*'] = TYPE_UINT8
 
+# GParamFlags
+G_PARAM_READABLE = 1 << 0
+G_PARAM_WRITABLE = 1 << 1
+G_PARAM_CONSTRUCT = 1 << 2
+G_PARAM_CONSTRUCT_ONLY = 1 << 3
+G_PARAM_LAX_VALIDATION = 1 << 4
+G_PARAM_STATIC_NAME = 1 << 5
+G_PARAM_STATIC_NICK = 1 << 6
+G_PARAM_STATIC_BLURB = 1 << 7
+
 SYMBOL_BLACKLIST = [
     # These ones break GError conventions
     'g_simple_async_result_new_from_error',
@@ -49,6 +60,16 @@ SYMBOL_BLACKLIST_RE = [re.compile(x) for x in \
                            [r'\w+_marshal_[A-Z]+__', ]]
 
 
+class IntrospectionBinary(object):
+
+    def __init__(self, args, tmpdir=None):
+        self.args = args
+        if tmpdir is None:
+            self.tmpdir = tempfile.mkdtemp('', 'tmp-introspect')
+        else:
+            self.tmpdir = tmpdir
+
+
 class Unresolved(object):
 
     def __init__(self, target):
@@ -63,34 +84,23 @@ class GLibTransformer(object):
 
     def __init__(self, transformer, noclosure=False):
         self._transformer = transformer
-        self._transformer.set_container_types(['GList*', 'GSList*'],
-                                              ['GHashtable*'])
+        self._noclosure = noclosure
         self._namespace_name = None
         self._names = Names()
         self._uscore_type_names = {}
-        self._libraries = []
+        self._binary = None
+        self._get_type_functions = []
+        self._error_quark_functions = []
+        self._gtype_data = {}
         self._failed_types = {}
         self._boxed_types = {}
         self._private_internal_types = {}
-        self._noclosure = noclosure
         self._validating = False
 
     # Public API
 
-    def add_library(self, libname):
-        # For testing mainly.
-        libtool_libname = 'lib' + libname + '.la'
-        if os.path.exists(libname):
-            found_libname = os.path.abspath(libname)
-        elif os.path.exists(libtool_libname):
-            found_libname = extract_libtool(libtool_libname)
-        elif libname.endswith('.la'):
-            found_libname = extract_libtool(libname)
-        else:
-            found_libname = find_library(libname)
-        if not found_libname:
-            raise ValueError("Failed to find library: %r" % (libname, ))
-        self._libraries.append(ctypes.cdll.LoadLibrary(found_libname))
+    def set_introspection_binary(self, binary):
+        self._binary = binary
 
     def _print_statistics(self):
         nodes = list(self._names.names.itervalues())
@@ -117,6 +127,11 @@ class GLibTransformer(object):
         if namespace.name == 'GObject':
             del self._names.aliases['Type']
 
+        # Get all the GObject data by passing our list of get_type
+        # functions to the compiled binary
+
+        self._execute_binary()
+
         # Introspection is done from within parsing
 
         # Second pass: pair boxed structures
@@ -132,11 +147,11 @@ class GLibTransformer(object):
                 print "WARNING: DELETING node %s: %s" % (node.name, e)
                 self._remove_attribute(node.name)
             # associate GtkButtonClass with GtkButton
-            if isinstance(node, Struct):
-                self._pair_class_struct(node)
+            if isinstance(node, Record):
+                self._pair_class_record(node)
         for (ns, alias) in self._names.aliases.itervalues():
             self._resolve_alias(alias)
-
+        self._resolve_quarks()
         self._print_statistics()
         # Fourth pass: ensure all types are known
         if not self._noclosure:
@@ -166,6 +181,16 @@ class GLibTransformer(object):
             return node[1]
         return None
 
+    def _lookup_node(self, name):
+        if name in type_names:
+            return None
+        node = self._get_attribute(name)
+        if node is None:
+            node = self._transformer.get_names().names.get(name)
+            if node:
+                return node[1]
+        return node
+
     def _register_internal_type(self, type_name, node):
         self._names.type_names[type_name] = (None, node)
         uscored = to_underscores(type_name).lower()
@@ -178,18 +203,15 @@ class GLibTransformer(object):
         no_uscore_prefixed = (prefix + '_' + to_underscores(suffix)).lower()
         self._uscore_type_names[no_uscore_prefixed] = node
 
+    def _resolve_quarks(self):
+        for node in self._error_quark_functions:
+            short = node.symbol[:-len('_quark')]
+            enum = self._uscore_type_names.get(short)
+            if enum is not None:
+                enum.error_quark = node.symbol
+
     # Helper functions
 
-    def _type_from_gtype(self, type_id):
-        ctype = cgobject.type_name(type_id)
-        type_name = type_name_from_ctype(ctype)
-        type_name = type_name.replace('*', '')
-        type_name = self._resolve_type_name(type_name)
-
-        type = Type(type_name, ctype)
-        type._gtype = type_id
-        return type
-
     def _resolve_gtypename(self, gtype_name):
         try:
             return self._transformer.gtypename_to_giname(gtype_name,
@@ -197,33 +219,64 @@ class GLibTransformer(object):
         except KeyError, e:
             return Unresolved(gtype_name)
 
+    def _execute_binary(self):
+        in_path = os.path.join(self._binary.tmpdir, 'types.txt')
+        f = open(in_path, 'w')
+        # TODO: Introspect GQuark functions
+        for func in self._get_type_functions:
+            f.write(func)
+            f.write('\n')
+        f.close()
+        out_path = os.path.join(self._binary.tmpdir, 'dump.xml')
+
+        args = []
+        args.extend(self._binary.args)
+        args.append('--introspect-dump=%s,%s' % (in_path, out_path))
+
+        # Invoke the binary, having written our get_type functions to types.txt
+        try:
+            subprocess.check_call(args, stdout=sys.stdout, stderr=sys.stderr)
+        except subprocess.CalledProcessError, e:
+            raise SystemExit(e)
+        self._read_introspect_dump(out_path)
+
+        # Clean up temporaries
+        shutil.rmtree(self._binary.tmpdir)
+
+    def _read_introspect_dump(self, xmlpath):
+        from xml.etree.cElementTree import parse
+        tree = parse(xmlpath)
+        root = tree.getroot()
+        for child in root:
+            self._gtype_data[child.attrib['name']] = child
+        for child in root:
+            self._introspect_type(child)
+
     def _create_gobject(self, node):
         type_name = 'G' + node.name
         if type_name == 'GObject':
             parent_gitype = None
             symbol = 'intern'
-        else:
-            type_id = cgobject.type_from_name(type_name)
-            parent_type_name = cgobject.type_name(
-                cgobject.type_parent(type_id))
+        elif type_name == 'GInitiallyUnowned':
+            parent_type_name = 'GObject'
             parent_gitype = self._resolve_gtypename(parent_type_name)
-            symbol = to_underscores(type_name).lower() + '_get_type'
-        node = GLibObject(node.name, parent_gitype, type_name, symbol, True)
-        type_id = cgobject.TYPE_OBJECT
-        self._introspect_properties(node, type_id)
-        self._introspect_signals(node, type_id)
-        self._add_attribute(node)
-        self._register_internal_type(type_name, node)
+            symbol = 'g_initially_unowned_get_type'
+        gnode = GLibObject(node.name, parent_gitype, type_name, symbol, True)
+        gnode.fields.extend(node.fields)
+        self._add_attribute(gnode)
+        self._register_internal_type(type_name, gnode)
 
     # Parser
 
     def _parse_node(self, node):
         if isinstance(node, Enum):
             self._parse_enum(node)
+        elif isinstance(node, Bitfield):
+            self._parse_bitfield(node)
         elif isinstance(node, Function):
             self._parse_function(node)
-        elif isinstance(node, Struct):
-            self._parse_struct(node)
+        elif isinstance(node, Record):
+            self._parse_record(node)
         elif isinstance(node, Callback):
             self._parse_callback(node)
         elif isinstance(node, Alias):
@@ -244,6 +297,9 @@ class GLibTransformer(object):
     def _parse_enum(self, enum):
         self._add_attribute(enum)
 
+    def _parse_bitfield(self, enum):
+        self._add_attribute(enum)
+
     def _parse_constant(self, constant):
         self._add_attribute(constant)
 
@@ -257,6 +313,8 @@ class GLibTransformer(object):
                 return
         if self._parse_get_type_function(func):
             return
+        if self._parse_error_quark_function(func):
+            return
 
         self._add_attribute(func)
 
@@ -267,6 +325,8 @@ class GLibTransformer(object):
         if self._namespace_name == 'GLib':
             # No GObjects in GLib
             return False
+        if func.parameters:
+            return False
         # GType *_get_type(void)
         if func.retval.type.name not in ['Type',
                                          'GType',
@@ -275,29 +335,20 @@ class GLibTransformer(object):
             print ("Warning: *_get_type function returns '%r'"
                    ", not GObject.Type") % (func.retval.type.name, )
             return False
+
+        self._get_type_functions.append(symbol)
+        return True
+
+    def _parse_error_quark_function(self, func):
+        if not func.symbol.endswith('_error_quark'):
+            return False
         if func.parameters:
             return False
-
-        if not self._libraries:
-            print "Warning: No libraries loaded, cannot call %s" % (symbol, )
+        if func.retval.type.name not in ['GLib.Quark',
+                                         'GQuark']:
             return False
 
-        for library in self._libraries:
-            try:
-                func = getattr(library, symbol)
-                break
-            except AttributeError:
-                continue
-        else:
-            print 'Warning: could not find symbol: %s' % symbol
-            name = symbol.replace('_get_type', '')
-            self._failed_types[name] = True
-            return False
-
-        func.restype = cgobject.GType
-        func.argtypes = []
-        type_id = func()
-        self._introspect_type(type_id, symbol)
+        self._error_quark_functions.append(func)
         return True
 
     def _name_is_internal_gtype(self, giname):
@@ -308,6 +359,28 @@ class GLibTransformer(object):
         except KeyError, e:
             return False
 
+    def _parse_static_method(self, func):
+        components = func.symbol.split('_')
+        if len(components) < 2:
+            return None
+        target_klass = None
+        prefix_components = None
+        methname = None
+        for i in xrange(1, len(components)-1):
+            prefix_components = '_'.join(components[0:-i])
+            methname = '_'.join(components[-i:])
+            target_klass = self._uscore_type_names.get(prefix_components)
+            if target_klass and isinstance(target_klass, GLibObject):
+                break
+            target_klass = None
+        if not target_klass:
+            return None
+        self._remove_attribute(func.name)
+        func.name = methname
+        target_klass.static_methods.append(func)
+        func.is_method = True
+        return func
+
     def _parse_method(self, func):
         if not func.parameters:
             return False
@@ -357,19 +430,6 @@ class GLibTransformer(object):
                 return None
             prefix = func.symbol[:new_idx]
 
-        klass = None
-
-        def valid_matching_klass(tclass):
-            if tclass is None:
-                return False
-            elif isinstance(klass, (GLibEnum, GLibFlags)):
-                return False
-            elif not isinstance(tclass, (GLibObject, GLibBoxed,
-                                          GLibInterface)):
-                return False
-            else:
-                return True
-
         klass = self._uscore_type_names.get(prefix)
         if klass is None:
             #print "NOTE: No valid matching class for likely "+\
@@ -381,9 +441,15 @@ class GLibTransformer(object):
 
         # The _uscore_type_names member holds the plain GLibBoxed
         # object; we want to actually use the struct/record associated
-        if isinstance(klass, GLibBoxed):
-            name = self._transformer.remove_prefix(klass.type_name)
-            klass = self._get_attribute(name)
+        if isinstance(klass, (Record, Union)):
+            remove_prefix = klass.symbol
+        else:
+            remove_prefix = klass.type_name
+
+        name = self._transformer.remove_prefix(remove_prefix)
+        klass = self._get_attribute(name)
+        if klass is None:
+            return
 
         if not is_method:
             # Interfaces can't have constructors, punt to global scope
@@ -395,7 +461,7 @@ class GLibTransformer(object):
             # class from the prefix
             # But for now, ensure that constructor returns are always
             # the most concrete class
-            name = self._transformer.remove_prefix(klass.type_name)
+            name = self._transformer.remove_prefix(remove_prefix)
             func.retval.type = Type(name, func.retval.type.ctype)
 
         self._remove_attribute(func.name)
@@ -405,32 +471,35 @@ class GLibTransformer(object):
             # We don't need the "this" parameter
             del func.parameters[0]
             klass.methods.append(func)
+            func.is_method = True
         else:
             klass.constructors.append(func)
         return func
 
-    def _parse_struct(self, struct):
+    def _parse_record(self, record):
         # This is a hack, but GObject is a rather fundamental piece so.
         internal_names = ["Object", 'InitiallyUnowned']
         g_internal_names = ["G" + x for x in internal_names]
         if (self._namespace_name == 'GObject' and
-            struct.name in internal_names):
-            self._create_gobject(struct)
+            record.name in internal_names):
+            self._create_gobject(record)
             return
-        elif struct.name in g_internal_names:
+        elif record.name in g_internal_names:
             # Avoid duplicates
             return
-        node = self._names.names.get(struct.name)
+        node = self._names.names.get(record.name)
         if node is None:
-            self._add_attribute(struct, replace=True)
+            self._add_attribute(record, replace=True)
+            self._register_internal_type(record.symbol, record)
             return
         (ns, node) = node
-        node.fields = struct.fields[:]
+        node.fields = record.fields[:]
 
     def _parse_union(self, union):
         node = self._names.names.get(union.name)
         if node is None:
             self._add_attribute(union, replace=True)
+            self._register_internal_type(union.symbol, union)
             return
         (ns, node) = node
         node.fields = union.fields[:]
@@ -455,7 +524,7 @@ class GLibTransformer(object):
             return True
         return False
 
-    def _pair_class_struct(self, maybe_class):
+    def _pair_class_record(self, maybe_class):
         name = self._strip_class_suffix(maybe_class.name)
         if name == maybe_class.name:
             return
@@ -474,8 +543,7 @@ class GLibTransformer(object):
         name = self._resolve_type_name(name)
         resolved = self._transformer.remove_prefix(name)
         pair_class = self._get_attribute(resolved)
-        if pair_class and isinstance(pair_class,
-                                     (GLibObject, GLibInterface)):
+        if pair_class and isinstance(pair_class, GLibInterface):
             for field in maybe_class.fields[1:]:
                 pair_class.fields.append(field)
             return
@@ -488,157 +556,132 @@ class GLibTransformer(object):
 
     # Introspection
 
-    def _introspect_type(self, type_id, symbol):
-        fundamental_type_id = cgobject.type_fundamental(type_id)
-        if (fundamental_type_id == cgobject.TYPE_ENUM or
-            fundamental_type_id == cgobject.TYPE_FLAGS):
-            self._introspect_enum(fundamental_type_id, type_id, symbol)
-        elif fundamental_type_id == cgobject.TYPE_OBJECT:
-            self._introspect_object(type_id, symbol)
-        elif fundamental_type_id == cgobject.TYPE_INTERFACE:
-            self._introspect_interface(type_id, symbol)
-        elif fundamental_type_id == cgobject.TYPE_BOXED:
-            self._introspect_boxed(type_id, symbol)
-        elif fundamental_type_id == cgobject.TYPE_BOXED:
-            self._introspect_boxed(type_id, symbol)
-        elif fundamental_type_id == cgobject.TYPE_POINTER:
-            # FIXME: Should we do something about these?
-            #        GHashTable, GValue and a few other fundamentals are
-            #        covered here
-            return
+    def _introspect_type(self, xmlnode):
+        if xmlnode.tag in ('enum', 'flags'):
+            self._introspect_enum(xmlnode)
+        elif xmlnode.tag == 'class':
+            self._introspect_object(xmlnode)
+        elif xmlnode.tag == 'interface':
+            self._introspect_interface(xmlnode)
+        elif xmlnode.tag == 'boxed':
+            self._introspect_boxed(xmlnode)
         else:
-            print 'unhandled GType: %s(%d)' % (cgobject.type_name(type_id),
-                                               type_id)
-
-    def _introspect_enum(self, ftype_id, type_id, symbol):
-        type_class = cgobject.type_class_ref(type_id)
-        if type_class is None:
-            return
+            raise ValueError("Unhandled introspection XML tag %s", xmlnode.tag)
 
+    def _introspect_enum(self, node):
         members = []
-        for enum_value in type_class.get_values():
-            members.append(GLibEnumMember(enum_value.value_nick,
-                                          enum_value.value,
-                                          enum_value.value_name,
-                                          enum_value.value_nick))
+        for member in node.findall('member'):
+            members.append(GLibEnumMember(member.attrib['nick'],
+                                          member.attrib['value'],
+                                          member.attrib['name'],
+                                          member.attrib['nick']))
 
-        klass = (GLibFlags if ftype_id == cgobject.TYPE_FLAGS else GLibEnum)
-        type_name = cgobject.type_name(type_id)
+        klass = (GLibFlags if node.tag == 'flags' else GLibEnum)
+        type_name = node.attrib['name']
         enum_name = self._transformer.remove_prefix(type_name)
-        node = klass(enum_name, type_name, members, symbol)
+        node = klass(enum_name, type_name, members, node.attrib['get-type'])
         self._add_attribute(node, replace=True)
         self._register_internal_type(type_name, node)
 
-    def _introspect_object(self, type_id, symbol):
-        type_name = cgobject.type_name(type_id)
+    def _introspect_object(self, xmlnode):
+        type_name = xmlnode.attrib['name']
         # We handle this specially above; in 2.16 and below there
         # was no g_object_get_type, for later versions we need
         # to skip it
         if type_name == 'GObject':
             return
-        parent_type_name = cgobject.type_name(cgobject.type_parent(type_id))
+        parent_type_name = xmlnode.attrib['parent']
         parent_gitype = self._resolve_gtypename(parent_type_name)
-        is_abstract = cgobject.type_is_abstract(type_id)
+        is_abstract = not not xmlnode.attrib.get('abstract', False)
         node = GLibObject(
             self._transformer.remove_prefix(type_name),
             parent_gitype,
             type_name,
-            symbol, is_abstract)
-        self._introspect_properties(node, type_id)
-        self._introspect_signals(node, type_id)
-        self._introspect_implemented_interfaces(node, type_id)
+            xmlnode.attrib['get-type'], is_abstract)
+        self._introspect_properties(node, xmlnode)
+        self._introspect_signals(node, xmlnode)
+        self._introspect_implemented_interfaces(node, xmlnode)
 
-        # add struct fields
-        struct = self._get_attribute(node.name)
-        if struct is not None:
-            node.fields = struct.fields
+        # add record fields
+        record = self._get_attribute(node.name)
+        if record is not None:
+            node.fields = record.fields
             for field in node.fields:
                 if isinstance(field, Field):
                     # Object instance fields are assumed to be read-only
-                    # (see also _pair_class_struct and transformer.py)
+                    # (see also _pair_class_record and transformer.py)
                     field.writable = False
 
         self._add_attribute(node, replace=True)
         self._register_internal_type(type_name, node)
 
-    def _introspect_interface(self, type_id, symbol):
-        type_name = cgobject.type_name(type_id)
-        parent_type_name = cgobject.type_name(cgobject.type_parent(type_id))
-        if parent_type_name == 'GInterface':
-            parent_gitype = None
-        else:
-            parent_gitype = self._resolve_gtypename(parent_type_name)
+    def _introspect_interface(self, xmlnode):
+        type_name = xmlnode.attrib['name']
         node = GLibInterface(
             self._transformer.remove_prefix(type_name),
-            parent_gitype,
-            type_name, symbol)
-        self._introspect_properties(node, type_id)
-        self._introspect_signals(node, type_id)
+            None,
+            type_name, xmlnode.attrib['get-type'])
+        self._introspect_properties(node, xmlnode)
+        self._introspect_signals(node, xmlnode)
+        for child in xmlnode.findall('prerequisite'):
+            name = child.attrib['name']
+            prereq = self._resolve_gtypename(name)
+            node.prerequisites.append(prereq)
         # GtkFileChooserEmbed is an example of a private interface, we
         # just filter them out
-        if symbol.startswith('_'):
+        if xmlnode.attrib['get-type'].startswith('_'):
             print "NOTICE: Marking %s as internal type" % (type_name, )
             self._private_internal_types[type_name] = node
         else:
             self._add_attribute(node, replace=True)
             self._register_internal_type(type_name, node)
 
-    def _introspect_boxed(self, type_id, symbol):
-        type_name = cgobject.type_name(type_id)
+    def _introspect_boxed(self, xmlnode):
+        type_name = xmlnode.attrib['name']
         # This one doesn't go in the main namespace; we associate it with
         # the struct or union
-        node = GLibBoxed(type_name, symbol)
+        node = GLibBoxed(type_name, xmlnode.attrib['get-type'])
         self._boxed_types[node.type_name] = node
         self._register_internal_type(type_name, node)
 
-    def _introspect_implemented_interfaces(self, node, type_id):
-        fundamental_type_id = cgobject.type_fundamental(type_id)
-        if fundamental_type_id != cgobject.TYPE_OBJECT:
-            raise AssertionError
-        interfaces = cgobject.type_interfaces(type_id)
+    def _introspect_implemented_interfaces(self, node, xmlnode):
         gt_interfaces = []
-        for interface_typeid in interfaces:
-            iname = cgobject.type_name(interface_typeid)
-            gitype = self._resolve_gtypename(iname)
+        for interface in xmlnode.findall('implements'):
+            gitype = self._resolve_gtypename(interface.attrib['name'])
             gt_interfaces.append(gitype)
         node.interfaces = gt_interfaces
 
-    def _introspect_properties(self, node, type_id):
-        fundamental_type_id = cgobject.type_fundamental(type_id)
-        if fundamental_type_id == cgobject.TYPE_OBJECT:
-            pspecs = cgobject.object_class_list_properties(type_id)
-        elif fundamental_type_id == cgobject.TYPE_INTERFACE:
-            pspecs = cgobject.object_interface_list_properties(type_id)
-        else:
-            raise AssertionError
-
-        for pspec in pspecs:
-            if pspec.owner_type != type_id:
-                continue
-            ctype = cgobject.type_name(pspec.value_type)
-            readable = (pspec.flags & 1) != 0
-            writable = (pspec.flags & 2) != 0
-            construct = (pspec.flags & 4) != 0
-            construct_only = (pspec.flags & 8) != 0
+    def _introspect_properties(self, node, xmlnode):
+        for pspec in xmlnode.findall('property'):
+            ctype = pspec.attrib['type']
+            flags = int(pspec.attrib['flags'])
+            readable = (flags & G_PARAM_READABLE) != 0
+            writable = (flags & G_PARAM_WRITABLE) != 0
+            construct = (flags & G_PARAM_CONSTRUCT) != 0
+            construct_only = (flags & G_PARAM_CONSTRUCT_ONLY) != 0
             node.properties.append(Property(
-                pspec.name,
+                pspec.attrib['name'],
                 type_name_from_ctype(ctype),
                 readable, writable, construct, construct_only,
                 ctype,
                 ))
 
-    def _introspect_signals(self, node, type_id):
-        for signal_info in cgobject.signal_list(type_id):
-            rtype = self._type_from_gtype(signal_info.return_type)
-            return_ = Return(rtype)
-            signal = GLibSignal(signal_info.signal_name, return_)
-            for i, parameter in enumerate(signal_info.get_params()):
+    def _introspect_signals(self, node, xmlnode):
+        for signal_info in xmlnode.findall('signal'):
+            rctype = signal_info.attrib['return']
+            rtype = Type(self._transformer.parse_ctype(rctype), rctype)
+            return_ = Return(rtype, signal_info.attrib['return'])
+            return_.transfer = PARAM_TRANSFER_FULL
+            signal = GLibSignal(signal_info.attrib['name'], return_)
+            for i, parameter in enumerate(signal_info.findall('param')):
                 if i == 0:
                     name = 'object'
                 else:
                     name = 'p%s' % (i-1, )
-                ptype = self._type_from_gtype(parameter)
+                pctype = parameter.attrib['type']
+                ptype = Type(self._transformer.parse_ctype(pctype), pctype)
                 param = Parameter(name, ptype)
+                param.transfer = 'none'
                 signal.parameters.append(param)
             node.signals.append(signal)
 
@@ -648,7 +691,6 @@ class GLibTransformer(object):
         # Workaround glib bug #548689, to be included in 2.18.0
         if type_name == "GParam":
             type_name = "GObject.ParamSpec"
-
         res = self._transformer.resolve_type_name_full
         try:
             return res(type_name, ctype, self._names)
@@ -656,6 +698,9 @@ class GLibTransformer(object):
             return self._transformer.resolve_type_name(type_name, ctype)
 
     def _resolve_param_type(self, ptype, **kwargs):
+        # Workaround glib bug #548689, to be included in 2.18.0
+        if ptype.name == "GParam":
+            ptype.name = "GObject.ParamSpec"
         return self._transformer.resolve_param_type_full(ptype,
                                                          self._names,
                                                          **kwargs)
@@ -670,21 +715,22 @@ class GLibTransformer(object):
             self._resolve_glib_object(node)
         elif isinstance(node, GLibInterface):
             self._resolve_glib_interface(node)
-        elif isinstance(node, Struct):
-            self._resolve_struct(node)
+        elif isinstance(node, Record):
+            self._resolve_record(node)
         elif isinstance(node, Union):
             self._resolve_union(node)
         elif isinstance(node, Alias):
             self._resolve_alias(node)
 
     def _resolve_function_toplevel(self, func):
-        newfunc = self._parse_constructor(func)
-        if not newfunc:
-            newfunc = self._parse_method(func)
-            if not newfunc:
-                self._resolve_function(func)
+        for parser in [self._parse_constructor,
+                       self._parse_method,
+                       self._parse_static_method]:
+            newfunc = parser(func)
+            if newfunc:
+                self._resolve_function(newfunc)
                 return
-        self._resolve_function(newfunc)
+        self._resolve_function(func)
 
     def _pair_boxed_type(self, boxed):
         name = self._transformer.remove_prefix(boxed.type_name)
@@ -692,7 +738,7 @@ class GLibTransformer(object):
         if not pair_node:
             boxed_item = GLibBoxedOther(name, boxed.type_name,
                                         boxed.get_type)
-        elif isinstance(pair_node, Struct):
+        elif isinstance(pair_node, Record):
             boxed_item = GLibBoxedStruct(pair_node.name, boxed.type_name,
                                          boxed.get_type)
             boxed_item.fields = pair_node.fields
@@ -704,7 +750,7 @@ class GLibTransformer(object):
             return False
         self._add_attribute(boxed_item, replace=True)
 
-    def _resolve_struct(self, node):
+    def _resolve_record(self, node):
         for field in node.fields:
             self._resolve_field(field)
 
@@ -735,6 +781,9 @@ class GLibTransformer(object):
         self._resolve_methods(node.methods)
         self._resolve_properties(node.properties, node)
         self._resolve_signals(node.signals)
+        node.prerequisites = filter(None,
+            [self._force_resolve(x, allow_unknown=True)
+             for x in node.prerequisites])
 
     def _resolve_glib_object(self, node):
         node.parent = self._force_resolve(node.parent)
@@ -743,6 +792,7 @@ class GLibTransformer(object):
                                     for x in node.interfaces])
         self._resolve_constructors(node.constructors)
         self._resolve_methods(node.methods)
+        self._resolve_methods(node.static_methods)
         self._resolve_properties(node.properties, node)
         self._resolve_signals(node.signals)
         for field in node.fields:
@@ -772,52 +822,13 @@ class GLibTransformer(object):
             except KeyError, e:
                 failed.append(prop)
         for fail in failed:
-            print ("WARNING: Deleting object property %r (of %r)" +\
-                       "with unknown type") % (fail, context)
+            print ("WARNING: Deleting object property %r (of %r) "
+                   "with unknown type") % (fail, context)
             properties.remove(fail)
 
     def _resolve_property(self, prop):
         prop.type = self._resolve_param_type(prop.type, allow_invalid=False)
 
-    def _adjust_transfer(self, param):
-        # Do GLib/GObject-specific type transformations here
-
-        transfer = None
-        is_object = None
-        if hasattr(param.type, '_gtype'):
-            ftype = cgobject.type_fundamental(param.type._gtype)
-            assert ftype != cgobject.TYPE_INVALID, param.type._gtype
-
-            is_object = ftype in [cgobject.TYPE_OBJECT,
-                                  cgobject.TYPE_INTERFACE]
-
-            if ftype in [cgobject.TYPE_OBJECT,
-                         cgobject.TYPE_INTERFACE,
-                         cgobject.TYPE_STRING,
-                         cgobject.TYPE_BOXED,
-                         cgobject.TYPE_PARAM]:
-                transfer = 'full'
-            else:
-                # if type is a cgobject.TYPE_POINTER we could require direction
-                # and transfer-ownership annotations
-                transfer = 'none'
-
-        if is_object is None:
-            is_object = (param.type.name == 'GObject.Object' or
-                         (self._namespace_name == 'GObject' and
-                          param.type.name == 'Object'))
-
-        # Default to full transfer for GObjects
-        if isinstance(param, Parameter):
-            is_out = (param.direction != PARAM_DIRECTION_IN)
-        else:
-            is_out = True
-        if param.transfer is None or param.transfer_inferred:
-            if is_out and is_object:
-                param.transfer = 'full'
-            elif transfer is not None:
-                param.transfer = transfer
-
     def _adjust_throws(self, func):
         if func.parameters == []:
             return
@@ -835,12 +846,10 @@ class GLibTransformer(object):
         self._resolve_parameters(func.parameters)
         func.retval.type = self._resolve_param_type(func.retval.type)
         self._adjust_throws(func)
-        self._adjust_transfer(func.retval)
 
     def _resolve_parameters(self, parameters):
         for parameter in parameters:
             parameter.type = self._resolve_param_type(parameter.type)
-            self._adjust_transfer(parameter)
 
     def _resolve_field(self, field):
         if isinstance(field, Callback):
