@@ -354,8 +354,6 @@ write_field_info (const gchar *namespace,
   if (size)
     xml_printf (file, " bits=\"%d\"", size);
 
-  xml_printf (file, " offset=\"%d\"", offset);
-
   type = g_field_info_get_type (info);
 
   if (branch)
@@ -457,6 +455,30 @@ write_callable_info (const gchar    *namespace,
       
       if (g_arg_info_is_optional (arg))
 	xml_printf (file, " optional=\"1\"");
+
+      switch (g_arg_info_get_scope (arg))
+        {
+        case GI_SCOPE_TYPE_INVALID:
+          break;
+        case GI_SCOPE_TYPE_CALL:
+          xml_printf (file, " scope=\"call\"");
+          break;
+        case GI_SCOPE_TYPE_OBJECT:
+          xml_printf (file, " scope=\"object\"");
+          break;
+        case GI_SCOPE_TYPE_ASYNC:
+          xml_printf (file, " scope=\"async\"");
+          break;
+        case GI_SCOPE_TYPE_NOTIFIED:
+          xml_printf (file, " scope=\"notified\"");
+          break;
+        }
+      
+      if (g_arg_info_get_closure (arg) >= 0)
+        xml_printf (file, " closure=\"%d\"", g_arg_info_get_closure (arg));
+      
+      if (g_arg_info_get_destroy (arg) >= 0)
+        xml_printf (file, " destroy=\"%d\"", g_arg_info_get_destroy (arg));
       
       type = g_arg_info_get_type (arg);
       write_type_info (namespace, type, file);
@@ -988,24 +1010,19 @@ write_interface_info (const gchar     *namespace,
 
   if (deprecated)
     xml_printf (file, " deprecated=\"1\"");
-	
 
   if (g_interface_info_get_n_prerequisites (info) > 0)
     {
-      xml_start_element (file, "requires");
       for (i = 0; i < g_interface_info_get_n_prerequisites (info); i++)
 	{
 	  GIBaseInfo *req = g_interface_info_get_prerequisite (info, i);
-	  
-	  if (g_base_info_get_type (req) == GI_INFO_TYPE_INTERFACE)
-            xml_start_element (file, "interface");
-	  else
-            xml_start_element (file, "object");
+
+	  xml_start_element (file, "prerequisite");
 	  write_type_name_attribute (namespace, req, "name", file);
+
           xml_end_element_unchecked (file);
 	  g_base_info_unref (req);
 	}
-      xml_end_element (file, "requires");
     }
 
   for (i = 0; i < g_interface_info_get_n_methods (info); i++)
